library(checkmate)

f = function(x) x[1]^2 + x[2]^2

kompasssuche2 = function(f, x0, s0, theta, opt = 0, tol = 0.001) {
  # check input:
  assert_function(f, args = "x")
  assert_numeric(x0, min.len = 1, finite = TRUE)
  assert_number(s0, finite = TRUE)
  assert_number(theta, lower = 0, upper = 1, finite = TRUE)
  assert_number(tol, lower = 0, finite = TRUE)

  # get possible directions:
  D = diag(length(x0))
  D = cbind(D, -D)
  
  # set counter to 0:
  k = 0
  
  # evaluate function with initial x0:
  fx0 = f(x0)
  
  # perform iterations until stopping criterion is reached:
  while(fx0 > abs(opt - tol)) {
    # initialize boolean for update:
    updated = FALSE
    d = 1
    
    # iterate over dimensions and check if improvement is achieved:
    while(d <= ncol(D) & !updated) {
      # calculate function value:
      val_d = f(x0 + s0 * D[, d])
      
      # check if there is an improvement:
      if(val_d < fx0) {
        # update x and f(x):
        x0 = x0 + s0 * D[, d] 
        fx0 = val_d
        
        # set updated to TRUE
        updated = TRUE
      }
      d = d + 1
    }
    # update s if no improvement is achieved:
    if(!updated) {
      s0 = theta * s0  
    }
    
    # increase counter:
    k = k + 1
    
    # check if too many iterations used:
    if(k >= 1000) stop("Optimum not found.")
  }
  # return number of iterations:
  return(k)
}
